%% MATLAB(R) Script for Replicating Abbring and Daljord (2020)
%
% This script computes the examples in Abbring and Daljord (2020). Its 
% results are stored in |.csv| and |.tex| files that are read by LaTeX upon 
% the paper's compilation to produce the graphs and documentation of the 
% paper's examples.
%
% Jaap H. Abbring and Øystein Daljord, May 2020
% 
%% Example in Which an Exclusion Restriction on Current Values Suffices for Identification but One on Primitive Utility Does Not
Qi = [0.25 0.25; 0 0.25];
QK = [0.9 0; 0 0.9; 0 1]; 
p = [0.5; 0.49; 0.1];

example(p,Qi,QK,'identificationFails',[1 2],'yes',[])

%% Example in Which Magnac and Thesmar (2002)'s Rank Condition Fails, but an Exclusion Restriction on Primitive Utility Suffices for Identification
% Use the same |Qi| and |QK| as in the previous example
p = [0.5; 0.48; 0.1];

example(p,Qi,QK,'moreEmpiricalContent',[1 2],'yes',[])

%% Example of Data that are Consistent with an Exclusion Restriction on Current Values but Not with One on Primitive Utility
Qi = [0.0 0.25; 0.25 0.25];
QK = [0 1;0 1;0 0];
p = [0.5; 0.48; 0.5];  

example(p,Qi,QK,'needNoMT',[1 2],'yes',[])

%% Example with Two Moment Conditions of Which One Identifies the Discount Factor
Qi =  [0.4000    0.3333    0.1905; 
       0.2647    0.2941    0.2647; 
       0.1786    0.3571    0.1786; 
       0.1818    0.2727    0.4545]; 

QK =  [0.1739    0.2609    0.1304; 
       0.1333    0.0667    0.2000; 
       0.2000    0.3000    0.1000; 
       0.2500    0.1500    0.5000]; 

% Impose $u_1(x_1) = u_1(x_2)$ and $u_1(x_3) = u_1(x_4)$
u1 = 0.5; 
u3 = 2; 
beta = 0.3; 
[~,~,~,p] = avf(QK,Qi,[u1 u1 u3 u3]',beta); 

example(p,Qi,QK,'oneRedundant',[1 2;3 4],'yes',[])

%% Example with Two Moment Conditions that Jointly Identify the Discount Factor but Individually Do Not
Qi =  [0.4286    0.3333    0.1905; 
       0.2647    0.2941    0.2647; 
       0.1786    0.3571    0.1786; 
       0.1818    0.2727    0.4545]; 

QK =  [0.1739    0.2609    0.1304; 
       0.1333    0.0667    0.2000; 
       0.2000    0.3000    0.1000; 
       0.2500    0.1500    0.5000]; 

% Impose $u_1(x_1) = u_1(x_2)$ and $u_1(x_3) = u_1(x_4)$
u1 = 2; 
u3 = 0.5;
beta = 0.9;  
[~,~,~,p] = avf(QK,Qi,[u1 u1 u3 u3]',beta) 

example(p,Qi,QK,'twiceTwo',[2 1;4 3],'yes',[])

%% Same Example with Noise in Choice Probabilities
rng(23670,'twister');
pHat = p + (1/1000)*rand(4,1)

example(pHat,Qi,QK,'twiceTwoNoise',[2 1;4 3],'yes',0.10)

%% Monotonicity Example

J = 3;
learnRate = 0.75;
Qi = diag((1-learnRate)*ones(J,1))+diag(learnRate*ones(J-1,1),1);
%Qi(end)=1;
Qi = Qi(:,1:end-1);
depRate = 0.5;
QK = diag((1-depRate)*ones(J,1))+diag(depRate*ones(J-1,1),-1);
QK(1)=1;
QK = QK(:,1:end-1);

beta=0.8;
u=@(x)-0.5+(x>2).*(x-2);
[~,~,~,p] = avf(QK,Qi,u(1:J)',beta) 
example(p,Qi,QK,'experience',[2 1],'yes',[])

name = 'experienceStructure'; 
% Save structural parameters to .tex file 
f1=fopen(['data//' name '.tex'],'w');
    fprintf(f1,['\\def\\' name 'learnRate{%6.2f}\n'],learnRate);
    fprintf(f1,['\\def\\' name 'depRate{%6.2f}\n'],depRate);
    fprintf(f1,['\\def\\' name 'beta{%6.2f}\n'],beta);
    fprintf(f1,['\\def\\' name 'uBoldPrime{\\left[\\begin{array}{cccc}%4.2f&%4.2f&%4.2f\\end{array}\\right]}\n'],u(1:J));
    fprintf(f1,['\\def\\' name 'uOne{%4.2f}\n'],u(1));
    fprintf(f1,['\\def\\' name 'uTwo{%4.2f}\n'],u(2));
    fprintf(f1,['\\def\\' name 'uThree{%4.2f}\n'],u(3));
fclose(f1);

%% Dependencies
%
% * <../example.m |example.m|>
% * <../avf.m |avf.m|>

%% References
%
% * Abbring, Jaap H. and Øystein Daljord (2020). Identifying the discount
% factor in dynamic discrete choice models. _Quantitative Economics 11(2)_, 471–501.
%
% * Thierry Magnac and David Thesmar (2002). Identifying dynamic discrete choice processes. _Econometrica 70_, 801-816.