function example(p,Qi,QK,name,iX,check,criticalValue)

%% EXAMPLE computes and plots example from Abbring and Daljord (2017) 
%
% EXAMPLE(p,Qi,QK,name,iX,check,criticalValue) writes .csv and .tex files
%   for an example specified by
%       p - J x (K-1) vector with choice probabilities
%       Qi - L x (K-1) matrix with first L<=K rows and K-1 columns of Q_i 
%       QK - K x (K-1) matrix with the first K-1 columns of Q_K
%       name - string, used for naming output files
%       iX - nMoments x 2 integer matrix with nMoments rows of state 
%            indices used for nMoments moment conditions
%       check - string, additional checks reported if 'yes'
%       criticalValue - scalar critical level for quadratic distance 
%   The number L of rows of Q_i needs to be no larger than the largest 
%   state index in iX (for example, one moment condition would typically 
%   compare x_1 and x_2 and only specify the first L=2 rows of Q_i)
%
%   Note that EXAMPLE 
%       adds columns to Qi and Qk with the residual probabilities;
%       compares the first with the K'th choice; 
%       computes and plots fairly general examples; but
%       only writes output files for specific examples with J=3 or 4.

%% Setup
warning('off','MATLAB:singularMatrix');
if exist('data')~=7;
        mkdir data;
end; % subfolder 'data' does not exist yet
disp(['Name: ' name]);
betas = linspace(0.001,0.999,200);      % 1 x 200 grid of beta's
J = size(p,1);                          % number of states
K = size(p,2)+1;                        % number of choices
nMoments = size(iX,1);                  % number of moment conditions
choicei = 1;                            % compare choicei=1 with K
momentExp = 2;                          % quadratic dist. x10^(2*momentExp)

% Construct full Qi and QK by adding residual probabilities 
Qi = [Qi 1-sum(Qi,2)]                   % L x K
QK = [QK 1-sum(QK,2)]                   % K x K

% Compute diff-in-diff of transition matrix Q (nMoments x J)
deltaQ = Qi(iX(:,1),:) - QK(iX(:,1),:) - Qi(iX(:,2),:)+ QK(iX(:,2),:)

% Add residual probability to p and compute excess surpluses   
p = [p 1-p]                             % J x K
m = -log(p(:,K))                        % J x 1
    

%% Compute, plot and solve moment conditions

% Left hand sides 
lhs = log(p(iX(:,1),choicei)./p(iX(:,1),K)...       
      .* p(iX(:,2),K)./p(iX(:,2),choicei))      % nMoments x 1

disp('Slope r.h.s. Magnac and Thesmar (2002):'); 
disp(deltaQ*m)

% Right hand sides
rhsData = rhs(betas,deltaQ,QK,m);               % nMoments x 200
rhsDataMT = deltaQ*m*betas;                     % nMoments x 200

% Cap rhsData to avoid TikZ collapse (the cap will not be displayed)
rhsData = (rhsData<=1).*rhsData + (rhsData>1); 
 
% moment distance criterion
quadDist = sum((10^momentExp*(rhsData-lhs)).^2,1);        % 1 x 200

% Plot right hand and left hand sides and quadratic distance
subplot(2,1,1); 
plot(betas,rhsData,'red',betas,lhs*ones(1,size(betas,2)),'black',...
         betas,rhsDataMT,'blue')
subplot(2,1,2);
plot(betas,quadDist,'blue')

% For each moment condition, solve for up to two betas that solve it
betaMT = lhs./(deltaQ*m)
beta1 = zeros(nMoments,1); 
beta2 = beta1;
options = optimset('Display','off');
for mm = 1:nMoments
    beta1(mm) = 1/(1+exp(fzero(@(b)rhs(1/(1+exp(b)),deltaQ(mm,:),...
                                                QK,m)-lhs(mm),4,options)));
    beta2(mm) = 1/(1+exp(fzero(@(b)rhs(1/(1+exp(b)),deltaQ(mm,:),...
                                               QK,m)-lhs(mm),-4,options)));   
end

% Solve for boundary points sets of betas with small enough quadr. distance 
if ~isempty(criticalValue)
    setEstim = (quadDist<=criticalValue);
    betaSet = sort([betas(1+find(setEstim(2:end)>setEstim(1:end-1)))...        
                    betas(1+find(setEstim(2:end)<setEstim(1:end-1)))]');
    for ss=1:size(betaSet,1)
        betaSet(ss) = 1/(1+exp(fzero(@(b)sum(((10^momentExp)*...
                      (rhs(1/(1+exp(b)),deltaQ,QK,m)-lhs)).^2,1)...
                     -criticalValue,log(1-betaSet(ss))-log(betaSet(ss)))));
    end
    betaSet
    subplot(2,1,2), plot(betas,quadDist,'black')
end % criticalValue

%% Checks on computations that are not necessary for the graphs
if check=='yes' 
    %vK1 = zeros(nMoments,J); 
    %vK2 = vK1; 
    %vKMT = vK1;
    %for mm = 1:nMoments
    %    vK1(mm,:) = inv(eye(J)/beta1(mm)-QK)*QK*m;
    %    vK2(mm,:) = inv(eye(J)/beta2(mm)-QK)*QK*m;
    %    vKMT(mm,:) = inv(eye(J)/betaMT(mm)-QK)*QK*m;
    %end
    %vK1
    %vK2
    %vKMT

    %disp('M&T rhs 1:'); disp(deltaQ*m.*beta1)
    %disp('vK rhs 1:'); disp(sum(deltaQ.*vK1.*beta1,2))
    %disp('M&T rhs 2:'); disp(deltaQ*m.*beta2)
    %disp('vK rhs 1:'); disp(sum(deltaQ.*vK2.*beta2,2))
    %disp('M&T rhs MT:'); disp(deltaQ*m.*betaMT)

    %disp('rhs 1'); disp(sum(beta1.*deltaQ.*(m'+vK1),2));
    %disp('rhs 2'); disp(sum(beta2.*deltaQ.*(m'+vK2),2));
    %disp('rhs MT'); disp(betaMT.*deltaQ*m);

    %viminvK =  log(p(:,choicei)./p(:,K))

    %ui1 = viminvK(iX) - beta1*(Qi-QK(iX,:))*(m+vK1)
    %ui2 = viminvK(iX) - beta2*(Qi-QK(iX,:))*(m+vK2)

    %capUi1 = ui1 + beta1*(Qi-QK(iX,:))*vK1
    %capUi2 = ui2 + beta2*(Qi-QK(iX,:))*vK2

    %capUiMT = viminvK(iX,:) - betaMT*(Qi-QK(iX,:))*m
    %uiMT = capUiMT - betaMT*(Qi-QK(iX,:))*vKMT
end % check

%% Export data for TikZ plotting
% Note: This export is specific to the paper's examples, with either
%       J=3 and one moment condition or J=4 and two moment conditions

if ((J==3)&(size(iX,1)==1))|((J==4)&(size(iX,1)==2)) 
 % Save plot data to .csv file 
 f1=fopen(['data//' name '.csv'],'w');
    if J==3 % one moment
        fprintf(f1,'beta, rhs, rhsMT, dummy\n');
        fprintf(f1,'%6.6f, %6.6f, %6.6f, 1.0\n',...
                                            [betas' rhsData' rhsDataMT']');
    elseif J==4 % two moments
        fprintf(f1,...
             'beta, oneRhs, twoRhs, oneRhsMT, twoRhsMT, quadDist, dummy\n');
        fprintf(f1,'%6.6f, %6.6f, %6.6f, %6.6f, %6.6f, %6.6f, 1.0\n',...
                     [betas' rhsData(1:2,:)' rhsDataMT(1:2,:)' quadDist']');
    end % J = 3 or 4
 fclose(f1);
 % Save auxiliary parameters to .tex file 
 f1=fopen(['data//' name '.tex'],'w');
    fprintf(f1,['\\def\\' name 'K{%2.0f}\n'],K);    
    fprintf(f1,['\\def\\' name 'J{%2.0f}\n'],J);    
    fprintf(f1,['\\def\\' name 'DistScaler{%2.0f}\n'],2*momentExp);    
    fprintf(f1,['\\def\\' name 'MinRhs{%6.4f}\n'],...
                                            min(min([rhsData rhsDataMT])));
    if J==3 % one moment
        fprintf(f1,['\\def\\' name 'PreciseLhs{%6.4f}\n'],lhs);      
        fprintf(f1,['\\def\\' name 'PreciseBetaOne{%6.2f}\n'],beta1);
        fprintf(f1,['\\def\\' name 'PreciseBetaTwo{%6.2f}\n'],beta2);
        fprintf(f1,['\\def\\' name 'PreciseBetaMT{%6.2f}\n'],betaMT);
    
        fprintf(f1,['\\def\\' name 'Lhs{%6.4f}\n'],lhs);      
        fprintf(f1,['\\def\\' name 'BetaOne{%6.2f}\n'],beta1);
        fprintf(f1,['\\def\\' name 'BetaTwo{%6.2f}\n'],beta2);
        fprintf(f1,['\\def\\' name 'BetaMT{%6.2f}\n'],betaMT);
    
        fprintf(f1,['\\def\\' name 'QiRowOne{\\left[\\begin{array}{ccc}%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],Qi(iX(1,1),:));
        fprintf(f1,['\\def\\' name 'QiRowTwo{\\left[\\begin{array}{ccc}%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],Qi(iX(1,2),:));
        fprintf(f1,['\\def\\' name 'QK{\\left[\\begin{array}{ccc}%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],QK');
        fprintf(f1,['\\def\\' name 'POne{\\left[\\begin{array}{ccc}%6.2f\\\\%6.2f\\\\%6.2f\\end{array}\\right]}\n'],p(:,1));    
        fprintf(f1,['\\def\\' name 'PK{\\left[\\begin{array}{ccc}%6.2f\\\\%6.2f\\\\%6.2f\\end{array}\\right]}\n'],p(:,K));
        fprintf(f1,['\\def\\' name 'MPrime{\\left[\\begin{array}{ccc}%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],m);
        fprintf(f1,['\\def\\' name 'DeltaQ{\\left[\\begin{array}{ccc}%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],deltaQ(1,:));
        fprintf(f1,['\\def\\' name 'InlineDeltaQ{$[%4.2f$~$%4.2f$~$%4.2f$]}\n'],deltaQ(1,:));

        fprintf(f1,['\\def\\' name 'DeltaQM{%6.4f}\n'],deltaQ*m);
    elseif J==4 % two moments
        fprintf(f1,['\\def\\' name 'OnePreciseLhs{%6.4f}\n'],lhs(1));      
        fprintf(f1,['\\def\\' name 'TwoPreciseLhs{%6.4f}\n'],lhs(2));      
        fprintf(f1,['\\def\\' name 'OnePreciseBetaOne{%6.2f}\n'],beta1(1));
        fprintf(f1,['\\def\\' name 'TwoPreciseBetaOne{%6.2f}\n'],beta1(2));
        fprintf(f1,['\\def\\' name 'OnePreciseBetaTwo{%6.2f}\n'],beta2(1));
        fprintf(f1,['\\def\\' name 'TwoPreciseBetaTwo{%6.2f}\n'],beta2(2));
        fprintf(f1,['\\def\\' name 'OnePreciseBetaMT{%6.2f}\n'],betaMT(1));
        fprintf(f1,['\\def\\' name 'TwoPreciseBetaMT{%6.2f}\n'],betaMT(2));

        fprintf(f1,['\\def\\' name 'OneLhs{%6.4f}\n'],lhs(1));      
        fprintf(f1,['\\def\\' name 'TwoLhs{%6.4f}\n'],lhs(2));      
        fprintf(f1,['\\def\\' name 'OneBetaOne{%6.2f}\n'],beta1(1));
        fprintf(f1,['\\def\\' name 'TwoBetaOne{%6.2f}\n'],beta1(2));
        fprintf(f1,['\\def\\' name 'OneBetaTwo{%6.2f}\n'],beta2(1));
        fprintf(f1,['\\def\\' name 'TwoBetaTwo{%6.2f}\n'],beta2(2));
        fprintf(f1,['\\def\\' name 'OneBetaMT{%6.2f}\n'],betaMT(1));
        fprintf(f1,['\\def\\' name 'TwoBetaMT{%6.2f}\n'],betaMT(2));
        fprintf(f1,['\\def\\' name 'Qi{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],Qi);
        fprintf(f1,['\\def\\' name 'QK{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\\\%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],QK');
        fprintf(f1,['\\def\\' name 'POne{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],p(:,1));    
        fprintf(f1,['\\def\\' name 'PK{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],p(:,K));
        fprintf(f1,['\\def\\' name 'MPrime{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],m);
        fprintf(f1,['\\def\\' name 'OneDeltaQ{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],deltaQ(1,:));
        fprintf(f1,['\\def\\' name 'TwoDeltaQ{\\left[\\begin{array}{cccc}%6.2f&%6.2f&%6.2f&%6.2f\\end{array}\\right]}\n'],deltaQ(2,:));
    
        fprintf(f1,['\\def\\' name 'OneDeltaQM{%6.4f}\n'],deltaQ(1,:)*m);
        fprintf(f1,['\\def\\' name 'TwoDeltaQM{%6.4f}\n'],deltaQ(2,:)*m);
    end %J=3 or 4
    if ~isempty(criticalValue)
        fprintf(f1,['\\def\\' name 'PreciseCriticalValue{%6.4f}\n'],criticalValue);
        fprintf(f1,['\\def\\' name 'PreciseBetaSetOne{%6.2f}\n'],betaSet(1));
        fprintf(f1,['\\def\\' name 'PreciseBetaSetTwo{%6.2f}\n'],betaSet(2));
        fprintf(f1,['\\def\\' name 'PreciseBetaSetThree{%6.2f}\n'],betaSet(3));
        fprintf(f1,['\\def\\' name 'PreciseBetaSetFour{%6.2f}\n'],betaSet(4));

        fprintf(f1,['\\def\\' name 'CriticalValue{%6.2f}\n'],criticalValue);  
        fprintf(f1,['\\def\\' name 'BetaSetOne{%6.2f}\n'],betaSet(1));
        fprintf(f1,['\\def\\' name 'BetaSetTwo{%6.2f}\n'],betaSet(2));
        fprintf(f1,['\\def\\' name 'BetaSetThree{%6.2f}\n'],betaSet(3));
        fprintf(f1,['\\def\\' name 'BetaSetFour{%6.2f}\n'],betaSet(4));
    end
  fclose(f1);
end % if dimension are those of paper's examples
end % function example

%% Auxiliary function to compute right hand side of moment condition
 function y = rhs(b,deltaQ,QK,m)  
        y=ones(size(deltaQ,1),size(b,2));
        for jj = 1:size(b,2)
            A = eye(size(QK,1))*1/b(jj) - QK; 
            y(:,jj) = deltaQ*(A\m);
        end
 end